@@ -5,7 +5,8 @@ df_match_variable <- structure(function # First match from every row, variable a
 ### ... -- argument names are interpreted as column names of subject;
 ### argument values are passed as the pattern to
 ### str_match_variable.
-(...
+(# can NOT have subject arg outside of dots (column named subject)
+  ...
 ### subject.df, colName1=list(groupName1=pattern1, fun1, etc),
 ### colName2=list(etc), etc. First (un-named) argument should be a
 ### data.frame with character columns of subjects for matching. The
@@ -26,7 +27,11 @@ df_match_variable <- structure(function # First match from every row, variable a
     stop("subject must be a data.frame with character columns to match")
   }
   col.pattern.list <- all.arg.list[-1]
-  if(length(col.pattern.list)==0)stop("no patterns specified in ...")
+  if(length(col.pattern.list)==0){
+    stop(
+      "no patterns specified in ...; ",
+      "must specify subjectColName=list(groupName=pattern, etc), etc")
+  }
   valid.name <- names(col.pattern.list) %in% names(subject)
   invalid.vec <- names(col.pattern.list)[!valid.name]
   if(is.null(names(col.pattern.list)) || length(invalid.vec)){
@@ -34,38 +39,51 @@ df_match_variable <- structure(function # First match from every row, variable a
          ") not found in subject column names (", paste(names(subject), collapse=", "),
          "); each pattern in ... must be named using a column name of subject")
   }
-  out.list <- list(subject)
-  has.rownames <- logical()
-  default.rownames <- paste(1:nrow(subject))
-  subject.names <- if(identical(rownames(subject), default.rownames)){
-    NULL
-  }else{
-    rownames(subject)
+  if(names(all.arg.list)[[1]] != ""){
+    stop("first argument (subject data.frame) should not be named")
   }
-  for(col.name in names(col.pattern.list)){
-    subject.vec <- subject[[col.name]]
-    names(subject.vec) <- subject.names
-    arg.list <- list(subject.vec, col.pattern.list[[col.name]])
-    result <- do.call(str_match_variable, arg.list)
-    group.names <- colnames(result)
-    out <- data.frame(result, stringsAsFactors=FALSE)
-    has.rownames[[col.name]] <- !identical(rownames(out), default.rownames)
-    names(out) <- paste0(col.name, ".", group.names)
-    out.list[[col.name]] <- out
-  }
-  if(is.null(subject.names) && 1 < sum(has.rownames)){
+  name.tab <- table(names(col.pattern.list))
+  if(any(bad <- 1 < name.tab)){
     stop(
-      "only one group named 'name' is allowed; problems: ",
-      paste(names(has.rownames)[has.rownames], collapse=", "))
+      "each argument name should be unique, problems: ",
+      paste(names(name.tab)[bad], collapse=", "))
   }
-  names(out.list) <- NULL
-  do.call(cbind, out.list)
+  out <- subject
+  name.group.used <- FALSE
+  for(col.name in names(col.pattern.list)){
+    subject.names <- if(.row_names_info(subject) > 0L){
+      attr(subject, "row.names")
+    }
+    subject.vec <- structure(subject[[col.name]], names=subject.names)
+    col.arg.list <- c(list(subject.vec), col.pattern.list[[col.name]])
+    m <- do.call(str_match_variable, col.arg.list)
+    has.names <- (
+      is.matrix(m) && !is.null(rownames(m))
+    ) || (
+      is.data.frame(m) && .row_names_info(m) > 0L
+    )
+    if(is.null(subject.names) && has.names){
+      if(name.group.used){
+        stop("only one group named 'name' is allowed")
+      }else{
+        name.group.used <- TRUE
+      }
+    }
+    if(is.character(colnames(m))){
+      colnames(m) <- paste0(col.name, ".", colnames(m))
+    }
+    out <- cbind(out, m, stringsAsFactors=FALSE)
+  }
+  out
 ### data.frame with same number of rows as subject, with an additional
 ### column for each named capture group specified in ...  (actually
 ### the value is created via base::cbind so if subject is something else
 ### like a data.table::data.table then the value is too).
 }, ex=function(){
 
+  ## The JobID column can be match with a complicated regular
+  ## expression, that we will build up from small sub-pattern list
+  ## variables that are easy to understand independently.
   (sacct.df <- data.frame(
     JobID = c(
       "13937810_25", "13937810_25.batch",
@@ -75,33 +93,48 @@ df_match_variable <- structure(function # First match from every row, variable a
       "00:00:00", "00:00:00"),
     stringsAsFactors=FALSE))
 
+  ## Just match the end of the range.
   int.pattern <- list("[0-9]+", as.integer)
+  end.pattern <- list(
+    "-",
+    task_end=int.pattern)
+  namedCapture::df_match_variable(sacct.df, JobID=end.pattern)
+
+  ## Match the whole range inside square brackets.
   range.pattern <- list(
     "[[]",
-    task1=int.pattern,
-    "(?:-",#begin optional end of range.
-    taskN=int.pattern,
-    ")?", #end is optional.
+    task_start=int.pattern,
+    end.pattern, "?", #end is optional.
     "[]]")
   namedCapture::df_match_variable(sacct.df, JobID=range.pattern)
 
+  ## Match either a single task ID or a range, after an underscore.
   task.pattern <- list(
     "_",
-    "(?:",#begin alternate
-    task=int.pattern,
-    "|",#either one task(above) or range(below)
-    range.pattern,
-    ")")#end alternate
+    list(
+      task_id=int.pattern,
+      "|",#either one task(above) or range(below)
+      range.pattern))
   namedCapture::df_match_variable(sacct.df, JobID=task.pattern)
 
+  ## Match type suffix alone.
+  type.pattern <- list(
+    "[.]",
+    type=".*")
+  namedCapture::df_match_variable(sacct.df, JobID=type.pattern)
+
+  ## Match task and optional type suffix.
+  task.type.pattern <- list(
+    task.pattern,
+    type.pattern, "?")
+  namedCapture::df_match_variable(sacct.df, JobID=task.type.pattern)
+
+  ## Match full JobID and Elapsed columns.
   (task.df <- namedCapture::df_match_variable(
     sacct.df,
     JobID=list(
       job=int.pattern,
-      task.pattern,
-      "(?:[.]",
-      type=".*",
-      ")?"),
+      task.type.pattern),
     Elapsed=list(
       hours=int.pattern,
       ":",
