@@ -2,7 +2,7 @@ engine <- structure(function
 ### Get current regex engine used by str_match_named and
 ### str_match_all_named. RE2 is used by default if the re2r package is
 ### available; otherwise, PCRE is used by default. The user can set
-### \code{options(engine="PCRE")} to use PCRE even when
+### \code{options(namedCapture.engine="PCRE")} to use PCRE even when
 ### RE2 is available.
 (){
   RE2.available <- requireNamespace("re2r", quietly=TRUE)
@@ -18,7 +18,7 @@ engine <- structure(function
 }, ex=function(){
 
   namedCapture::engine()
-  old.opt <- options(engine="PCRE")
+  old.opt <- options(namedCapture.engine="PCRE")
   namedCapture::engine()
   options(old.opt)
 
